#include "httpd.h"


ServerURL::ServerURL(const std::string& url)
{
    unparsed = url;
    int qPos = unparsed.find('?');
    if (qPos >= 0)
    {
        path = unescapeStr(unparsed.substr(0, qPos));
        unparsedOptions = unparsed.substr(qPos + 1);
    }
    else
        path = unparsed;
    if (unparsedOptions != "")
        parseOptions();
    
}

ServerURL::~ServerURL()
{
}

const std::string& ServerURL::getOptions() const
{
    return unparsedOptions;
}

const std::string& ServerURL::getPath() const
{
    return path;
}

const std::string& ServerURL::getOption(const std::string& option, const std::string& dflt)
{
    std::string& s(options[option]);
    if (s == "")
        return dflt;
    else
        return s;
}

const int ServerURL::getOption(const std::string& option, int dflt)
{
    std::string s = options[option];
    if (s == "")
        return dflt;
    else
        return atoi(s.c_str());
}

const double ServerURL::getOption(const std::string& option, double dflt)
{
    std::string s = options[option];
    if (s == "")
        return dflt;
    else
        return atof(s.c_str());
}


std::string ServerURL::unescapeStr(const std::string& str)
{
    std::string res;
    int i = 0;
    int len = str.length();
    char sb[2], hb[3], *endptr;

    sb[1] = 0;
    hb[2] = 0;
    while (i < len)
    {
        if (str[i] == '%')
        {
            if (i + 2 >= len)
                throw HttpException(400, "Invalid URL");
            hb[0] = str[++i];
            hb[1] = str[++i];
            sb[0] = (char)strtol(hb, &endptr, 16);
            if (endptr != &hb[3])
                throw HttpException(400, "Invalid URL");
            res.append(sb);
        }
        else
        {
            sb[0] = str[i];
            res.append(sb);
        }
        i++;
    }
    return res;
}

void ServerURL::parseOptions()
{
    int i = 0;
    int len = unparsedOptions.length();

    int startName = 0;
    int startValue = -1;
    std::string name;
    std::string value;
    while (i < len)
    {
        if (unparsedOptions[i] == '&')
        {
            if (startValue < 0)
                name = unparsedOptions.substr(startName, i - startName);
            else
                value = unparsedOptions.substr(startValue, i - startValue);
            if (name != "")
                options[unescapeStr(name)] = unescapeStr(value);
            name = "";
            value = "";
            startName = i + 1;
            startValue = -1;
        }
        else if (unparsedOptions[i] == '=')
        {
            name = unparsedOptions.substr(startName, i - startName);
            startValue = i + 1;
        }
        i++;
    }
}

