#include <stdlib.h>
#include <sys/types.h>
#ifdef WIN32
#include <windows.h>
#else
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#endif
#include "httpd.h"
#include "exceptions.h"
#include "utils.h"



unsigned long int Subnet::atoip(const std::string& str)
{
    struct in_addr inp;
#ifndef WIN32
    int res;

    res = inet_aton(str.c_str(), &inp);
    if (! res)
#else
    inp.s_addr = inet_addr(str.c_str());
    if (inp.s_addr == INADDR_NONE)
#endif
        throw MasException("Invalid ip address '" + str + "'");

    return inp.s_addr;
}


unsigned long int Subnet::atoipmask(const std::string& str)
{
    struct in_addr inp;
    char *p;

    p = strchr(str.c_str(), '.');
    if (! p)
    {
        int bits;
        char *start;
        
        bits = strtol(str.c_str(), &start, 10);
        if (((! str[0]) || (start[0])) || ((bits < 0) || (bits > 32)))
            throw MasException("Invalid network mask '" + str + "'");
        
        return htonl(0xFFFFFFFF << (32 - bits));
    }
    else
    {
#ifndef WIN32
        int res = inet_aton(str.c_str(), &inp);
        if (! res)
#else
        inp.s_addr = inet_addr(str.c_str());
        if ((inp.s_addr == INADDR_NONE) && (str != "255.255.255.255"))
#endif
            throw MasException("Invalid network mask '" + str + "'");
    }

    return inp.s_addr;
}



Subnet::Subnet(const std::string& a, const std::string& m)
{
    if (a == "*")
    {
        addr = atoip("0.0.0.0");
        mask = atoipmask("0.0.0.0");
    }
    else
    {
        addr = atoip(a);
        if (m == "")
            mask = atoipmask("255.255.255.255");
        else
            mask = atoipmask(m);
    }
}


Subnet::Subnet(const std::string& a)
{
    if (a == "*")
    {
        addr = atoip("0.0.0.0");
        mask = atoip("0.0.0.0");
    }
    else
    {
        addr = atoip(a);
        mask = atoipmask("255.255.255.255");
    }
}


Subnet::Subnet(const Subnet& subnet)
{
    addr = subnet.addr;
    mask = subnet.mask;
}



Subnet::~Subnet()
{
}
        


unsigned long int Subnet::getAddress() const
{
    return addr;
}

        
unsigned long int Subnet::getMask() const
{
    return mask;
}

        
bool Subnet::containsAddress(unsigned long int a)
{
//std::cout << "checking " << toString() << " " << hex << addr << "/" << mask << " " << a << dec << std::endl;
    return ((a & mask) == addr);
}
        

bool Subnet::containsAddress(const std::string& a)
{
    unsigned long int ad = atoip(a);
    return containsAddress(ad);
}


std::string Subnet::toString()
{
    return std::string(inet_atos(addr) + "/" + inet_atos(mask));
}

