#ifndef __ARENA_H__
#define __ARENA_H__

#include <map>
#include <string>
#include <set>
#include "unit.h"
#include "link.h"


class COREIMPORT Arena
{
    private:
        std::map<std::string, Unit*> units;
        std::string filename;
    
        class Chain
        {
            public:
                Unit *from, *to;
                Chain(Unit *f, Unit *t) { from = f; to = t; };
                Chain(const Chain &c) { from = c.from; to = c.to; };
                bool operator == (const Chain &c) { return (c.from == from) && (c.to == to); };
                bool operator < (const Chain &c) { return (c.from != from) || (c.to != to); };
                friend bool operator < (const Chain &c1, const Chain &c2) { return (c1.from != c2.from) || (c1.to != c2.to); };
        };

        std::set<Chain> chains;
        
    public:
        Arena(const std::string &filename);
        ~Arena();

    public:
        void load();
        void save();

        void addUnit(Unit *unit);
        void delUnit(Unit *unit);
        
        void addLink(Unit *from, Unit *to);
        void addLink(const std::string &fromName, const std::string &toName);
        void delLink(Unit *from, Unit *to);

        std::string genUniqueName(const std::string &base);

    private:
        void loadUnits(HTable conf);
        void loadLinks(HTable conf);
};


#endif

