#ifndef __LINK_H__
#define __LINK_H__


#include <list>
#include "thread.h"
#include "mediatype.h"
#include "mediainfo.h"
#include "packetizer.h"


class COREIMPORT Link: public PacketizerReceiver
{
    private:
        typedef struct
        {
            int refCount;
            MediaType *mtype;
        } PacketMediaType;
        
        typedef struct
        {
            int refCount;
            MediaInfo *minfo;
        } PacketMediaInfo;
        
        typedef struct
        {
            int start;
            int length;
            double playTime;
            PacketMediaType *packetMediaType;
            PacketMediaInfo *packetMediaInfo;
        } Packet;

        unsigned char *buffer;
        Packet *packets;
        Lock lockBuffer;
        Cond notEmpty;
        int firstPacketIndex;
        int nextPacketIndex;
        int packetsDataBufferSize;
        int packetsInfoBufferSize;
        int packetsCount;
        unsigned long long int firstPacketNo;
        std::list<PacketMediaType*> packetsMediaTypes;
        PacketMediaType* lastPacketMediaType;
        std::list<PacketMediaInfo*> packetsMediaInfos;
        PacketMediaInfo* lastPacketMediaInfo;
        
    public:
        Link(int maxBufSize = 1024*128, int maxPacketsCount = 500);
        virtual ~Link();

    public:
        void setMediaInfo(const MediaInfo &mediaInfo);
        virtual void addData(unsigned char *data, int length, double playTime, 
                const MediaType &mediaType);
        void beginGet();
        void getPacketInfo(MediaType* &mediaType, MediaInfo* &mediaInfo, 
                int &length, double &playTime, unsigned long long int &packetNo);
        void getData(unsigned long long int &packetNo, unsigned char *data);
        void endGet();

    private:
        int correctPacketNo(unsigned long long int packetNo);
        void removeFirstPacket();
        int waitForPacket(unsigned long long int &packetNumber);
        void setEmptyMediaInfo();
};


#endif

