#include <time.h>
#include <stdio.h>
#include "log.h"
#include "exceptions.h"

#ifdef ENABLE_FILELOG
#include "log_file.h"
#endif
#ifdef ENABLE_STDOUTLOG
#include "log_stdout.h"
#endif
#ifdef ENABLE_SYSLOG
#include "log_syslog.h"
#endif



Log masLog;


static LogAgent* createAgent(const std::string& agentName, HTable conf);

        
std::string LogAgent::formatMessage(MessageType type, const std::string& msg)
{
    char buf[50];
#ifndef WIN32
    time_t t = time(NULL);
#ifdef sun
    ctime_r(&t, buf, 50);
#else // not sun
    ctime_r(&t, buf);
#endif // sun
    buf[24] = 0;

    std::string s(buf);
    s += " ";
#else // WIN32
    _strdate(buf);
    std::string s(buf);
    _strtime(buf);
    s = s + " " + buf + " ";
#endif // WIN32
    switch (type)
    {
        case T_ERROR: s += "ERROR: "; break;
        case T_WARNING: s += "WARNING: "; break;
        case T_DEBUG: s += "DEBUG: "; break;
        default: s += "INFO: ";
    }
    s += msg;
    return s;
}



Log::Log()
{
    agent = NULL;
}


Log::~Log()
{
    try
    {
        if (agent)
            delete agent;
    }
    catch (...)
    {
    }
}


void Log::error(const std::string& msg)
{
    try
    {
        if (agent)
            agent->logMessage(LogAgent::T_ERROR, msg);
        else
            printf("ERROR: %s\n", msg.c_str());
    }
    catch (...)
    {
    }
}


void Log::warn(const std::string& msg)
{
    try
    {
        if (agent)
            agent->logMessage(LogAgent::T_WARNING, msg);
        else
            printf("WARNING: %s\n", msg.c_str());
    }
    catch (...)
    {
    }
}


void Log::info(const std::string& msg)
{
    try
    {
        if (agent)
            agent->logMessage(LogAgent::T_INFO, msg);
        else
            printf("INFO: %s\n", msg.c_str());
    }
    catch (...)
    {
    }
}


void Log::debug(const std::string& msg)
{
    try
    {
        if (agent)
            agent->logMessage(LogAgent::T_DEBUG, msg);
        else
            printf("DEBUG: %s\n", msg.c_str());
    }
    catch (...)
    {
    }
}


void Log::setAgent(HTable conf)
{
    if (! conf)
    {
        error("Log parameters missing");
        return;
    }
    
    try
    {
        std::string type(table_get_strs(conf, "type", "", NULL));
        LogAgent *a = createAgent(type, conf);
        if (a)
        {
            if (agent)
            {
                try { delete agent; } catch (...) { }
            }
            agent = a;
        }
    }
    catch (MasException& e)
    {
        error(e.getMessage());
    }
    catch (...)
    {
        error("Error switching log agent");
    }
}


static LogAgent* createAgent(const std::string& agentName, HTable conf)
{
#ifdef ENABLE_FILELOG
    if (agentName == "file")
        return new FileLogAgent(conf);
#endif
#ifdef ENABLE_STDOUTLOG
    if (agentName == "stdout")
        return new StdoutLogAgent(conf);
#endif
#ifdef ENABLE_SYSLOG
    if (agentName == "syslog")
        return new SyslogLogAgent(conf);
#endif
    throw MasException("Invalid log agent '" + agentName + "'");
}

