#include "mediatype.h"


MediaType::MediaType(const std::string &type, const std::string &subtype)
{
    this->type = type;
    this->subtype = subtype;
}


MediaType::MediaType(const MediaType &mtype)
{
    type = mtype.type;
    subtype = mtype.subtype;
}


MediaType::~MediaType()
{
}


std::string MediaType::getType() const
{
    return type;
}


std::string MediaType::getSubtype() const
{
    return subtype;
}


bool MediaType::isEqual(const MediaType &mtype) const
{
    if ((mtype.type == type) && (mtype.subtype == subtype))
        return true;
    else
        return false;
}


MediaType* MediaType::clone() const
{
    return new MediaType(type, subtype);
}


std::string MediaType::getMimeType() const
{
    return type + "/" + subtype;
}


bool isSupportedMediaType(std::set<MediaType> &types, const MediaType &mediaType)
{
    for (std::set<MediaType>::iterator i = types.begin(); i != types.end(); i++)
    {
        bool sameType;
        MediaType m = *i;
        if ((m.getType() == "*") || (mediaType.getType() == "*"))
            sameType = true;
        else
            sameType = (m.getType() == mediaType.getType());
        if (sameType)
        {
            bool sameSubtype;
            if ((m.getSubtype() == "*") || (mediaType.getSubtype() == "*"))
                sameSubtype = true;
            else
                sameSubtype = (m.getSubtype() == mediaType.getSubtype());
            if (sameSubtype)
                return true;
        }
    }
    return false;
}


COREIMPORT bool operator < (const MediaType &m1, const MediaType &m2)
{
    std::string s1(m1.getType() + "/" + m1.getSubtype());
    std::string s2(m2.getType() + "/" + m2.getSubtype());
    return s1 < s2;
}

