#include <unistd.h>
#include <sys/types.h>
#ifndef WIN32
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#else
#include <winsock2.h>
#include <ws2tcpip.h>
#endif
#include "multisock.h"
#include "exceptions.h"
#include "log.h"


MulticastSocket::MulticastSocket(HTable params, bool output)
{
    struct ip_mreq mreqn;
    int arg;
    
    memset(&mreqn, 0, sizeof(struct ip_mreq));
    
    sock = socket(AF_INET, SOCK_DGRAM, 0);
    if (sock < 0)
        throw MasException("Error creating socket.  Is TCP/IP installed? ;-)");

    arg = 1;
    if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (char*)&arg, sizeof(int)))
        masLog.warn("Error setting SO_REUSEADR");
    unsigned char ttl = (unsigned char)table_get_int(params, "ttl", 1, NULL);
    if (ttl > 0)
    {
        if (setsockopt(sock, IPPROTO_IP, IP_MULTICAST_TTL, (char*)&ttl, sizeof(char)))
            masLog.warn("Error setting IP_MULTICAST_TTL"); 
    }
    char loop = (unsigned char)table_get_int(params, "loop", 1, NULL);
    if (setsockopt(sock, IPPROTO_IP, IP_MULTICAST_LOOP, &loop, sizeof(char)))
        masLog.warn("Error setting IP_MULTICAST_LOOP");
    
    sin.sin_family = AF_INET;
    sin.sin_port = htons(table_get_int(params, "port", 0, NULL));
    addrLen = sizeof(struct sockaddr_in);
    std::string multAddr = std::string(table_get_strs(params, "address", "224.1.0.1", NULL));
   /* if (multAddr != "")
        sin.sin_addr.s_addr = inet_addr(multAddr.c_str());
    else*/
        sin.sin_addr.s_addr = INADDR_ANY;
    
    if (bind(sock, (struct sockaddr*)&sin, addrLen) < 0)
    {
        shutdown(sock, 2);
        close(sock);
        throw MasException("Error binding multicast socket");
    } 
    
    std::string bindAddr = std::string(table_get_strs(params, "bindaddress", "", NULL));
    if (bindAddr != "")
    {
        struct sockaddr_in stLocal;
        memset(&stLocal, 0, sizeof(stLocal));
        stLocal.sin_addr.s_addr = inet_addr(bindAddr.c_str());
        int ret = setsockopt(sock, IPPROTO_IP, IP_MULTICAST_IF,
                     (char*)&stLocal.sin_addr.s_addr, 
                     sizeof(stLocal.sin_addr.s_addr));
        if (ret < 0)
            masLog.warn("Error setting IP_MULTICAST_IF");
    }
    
    mreqn.imr_multiaddr.s_addr = inet_addr(multAddr.c_str());
    if (output)
    {
        mreqn.imr_interface.s_addr = sin.sin_addr.s_addr;
    }
    else
    {
        if (bindAddr != "")
            mreqn.imr_interface.s_addr = inet_addr(bindAddr.c_str());
        else
            mreqn.imr_interface.s_addr = INADDR_ANY;
    }

    if (setsockopt(sock, IPPROTO_IP, IP_ADD_MEMBERSHIP, (char*)&mreqn, sizeof(mreqn)) < 0)
    {
        shutdown(sock, 2);
        close(sock);
        throw MasException("Error joining to milticast group"); 
    }
    sin.sin_addr.s_addr = inet_addr(multAddr.c_str());
}


MulticastSocket::~MulticastSocket()
{
    shutdown(sock, 2);
    close(sock);
}
    
int MulticastSocket::read(unsigned char* buf, int max_size)
{
    int received = recvfrom(sock, (char*)buf, max_size, 0, NULL, NULL);
    return received;
}

int MulticastSocket::write(unsigned char* buf, int size)
{
    int sended  = sendto(sock, (char*)buf, size, 0, (struct sockaddr*)&sin, addrLen);
    return sended;
}

