#ifndef __RANDOM_H__
#define __RANDOM_H__

#include <pthread.h>
#include "corelibcfg.h"


class COREIMPORT Random
{
    private:
#define RAND_N 624
        unsigned long mt[RAND_N]; /* the array for the state vector  */
        int mti; /* mti==RAND_N+1 means mt[RAND_N] is not initialized */
        bool synchronized;
        pthread_mutex_t lock;
    
    public:
        Random(bool synchronized = true);
        Random(unsigned long int seed, bool synchronized = true);
        Random(int keys[], int length, bool synchronized = true);
        ~Random();

    public:
        /* generates a random number on [0,0xffffffff]-interval */
        unsigned long int genInt32();
        /* generates a random number on [0,0x7fffffff]-interval */
        long int genInt31();
        /* generates a random number on [0,1]-real-interval */
        double genReal1(); 
        /* generates a random number on [0,1)-real-interval */
        double genReal2(); 
        /* generates a random number on (0,1)-real-interval */
        double genReal3(); 
        /* generates a random number on [0,1) with 53-bit resolution*/
        double genReal53(); 

    private:
        void createLock();
        void initLong(unsigned long int s);
};


extern COREIMPORT Random rndGen;


#endif

