#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include "selector.h"
#include "utils.h"


Selector::Selector()
{
    FD_ZERO(&fdset);
    maxDescriptor = 0;
}

Selector::~Selector()
{
    FD_ZERO(&fdset);
}

void Selector::clear()
{
    FD_ZERO(&fdset);
    maxDescriptor = 0;
}

void Selector::addReadSocket(ReadSocket &sel)
{
    if (! sel.isReadSelectable())
        return;
    int descr = sel.getSelectorReadDescr();
    FD_SET(descr, &fdset);
    if (descr > maxDescriptor)
        maxDescriptor = descr;
}

int Selector::waitForEvent()
{
    int res = select(maxDescriptor + 1, &fdset, NULL, NULL, NULL);
    return res;
}

int Selector::waitForEvent(struct timeval& timeout)
{
    int res = select(maxDescriptor + 1, &fdset, NULL, NULL, &timeout);
    return res;
}

bool Selector::canRead(ReadSocket &sel)
{
    if (! sel.isReadSelectable())
        return false;
    int descr = sel.getSelectorReadDescr();
    if (FD_ISSET(descr, &fdset))
        return true;
    else
        return false;
}

