#ifndef __SERVER_H__
#define __SERVER_H__

#include <string>
#include <map>

#include "corelibcfg.h"
//#include "threadpool.h"
#include "plugin.h"
#include "unit.h"
#include "arena.h"
#include "packetizer.h"


class COREIMPORT Server: public Interruptable
{
    private:
        //ThreadPool *threadPool;
        std::map<std::string, Plugin*> plugins;
        std::map<std::string, RunnablePlugin*> runnablePlugins;
        std::map<std::string, Loader*> loaders;
        std::map<std::string, UnitFactory*> unitFactories;
        std::map<std::string, PacketizerFactory*> packetizerFactories;
        HTable config;
        Arena *arena;
        
    public:
        Server(const std::string& configFile);
        virtual ~Server();

    public:
        void run();
        static Server* getServer();
        //ThreadPool *getThreadPool();
        HTable getConfig();
        
        void addPlugin(const std::string& name, Plugin *plugin);
        Plugin* findPlugin(const std::string &name);
        
        void addRunnablePlugin(const std::string& name, RunnablePlugin *plugin);
        RunnablePlugin* findRunnablePlugin(const std::string &name);

        void addLoader(const std::string& name, Loader *loader);
        Loader* getLoader(const std::string &name);

        void addUnitFactory(const std::string& name, UnitFactory *factory);
        Unit* createUnit(const std::string &name, HTable conf);

        void addPacketizerFactory(const std::string& name, PacketizerFactory *factory);
        Packetizer* createPacketizer(const std::string &name, PacketizerReceiver *link=NULL);

    private:
        void loadModules();
};


#endif

