#ifndef __THREADS_H__
#define __THREADS_H__


#include "corelibcfg.h"

/*#ifdef WIN32
#include <windows.h>
#else*/
#include <pthread.h>
/*#endif*/


class Signal;


class COREIMPORT Lock
{
    friend class Cond;

    private:
/*#ifdef WIN32
        CRITICAL_SECTION mutex;
#else*/
        pthread_mutex_t mutex;
/*#endif*/
    
    public:
        Lock(bool locked = false);
        ~Lock();

    public:
        void lock();
        void unlock();
};


class COREIMPORT Cond
{
    private:
        pthread_cond_t cond;

    public:
        Cond();
        ~Cond();

    public:
        void wait(Lock &lock);
        void wait(Lock &lock, const struct timespec *abstime);
        void wait(Lock &lock, int sec);
        void signal();
        void broadcast();
};


class COREIMPORT Interruptable
{
    protected:
        Lock stopLock;
        Cond stopCond;
        bool stopThread;
    
    public:
        Interruptable();
        virtual ~Interruptable();

    public:
        virtual void stop();
        virtual bool isShuttingDown();
        void waitForShutdown();
        void waitForShutdown(int sec);
};


class COREIMPORT Thread: public Interruptable
{
    friend void* runThread(void *data);

    protected:
        pthread_t handle;
        bool detached;
        bool autoDelete;
        bool running;
    
    public:
        Thread();
        virtual ~Thread();

    public:
        virtual void start(bool daemon = false, bool autoDelete = false);
        virtual void stop(bool waitForTermination = true);
        virtual void run() = 0;
        virtual bool isRunning();
};


class COREIMPORT Locker
{
    private:
        Lock *lock;
        bool destroyIt;
    
    public:
        Locker();
        Locker(Lock& lock);
        ~Locker();
};


#endif

