#ifndef __THREADPOOL_H__
#define __THREADPOOL_H__


#include <list>
#include "thread.h"
#include "conf.h"


class COREIMPORT PoolWorker;


class COREIMPORT PoolJob: public Thread
{
    friend class PoolWorker;
    
    private:
        bool autoDestroy;
        PoolWorker *worker;
        Lock workerLock;
    
    public:
        PoolJob(bool autoDestroy = true);
        virtual ~PoolJob();

    public:
        virtual void run(void *data) = 0;
        virtual bool isShuttingDown();

    private:
        virtual bool destroyAfterWorkDone();
        virtual void run() { };
        virtual void run(PoolWorker *worker, void *data);
};


class COREIMPORT ThreadPool;


class COREIMPORT PoolWorker: public Thread
{
    private:
        PoolJob *job;
        void *data;
        Lock jobLock;
        Cond newJobCond;
        ThreadPool &pool;
        int idleTimeout;
    
    public:
        PoolWorker(ThreadPool &p, int idleTimeout);
        PoolWorker(ThreadPool &p, int idleTimeout, PoolJob *job, void *data);
        virtual ~PoolWorker();
       
    public:
        virtual void run();
        void setJob(PoolJob *job, void *data);
        virtual void stop(bool waitForTermination = true);
};


class COREIMPORT ThreadPool
{
    friend class PoolWorker;
    
    private:
        int maxThreads;
        int minThreads;
        int idleTimeout;
        int maxQueue;
        std::list<PoolWorker*> idleThreads;
        std::list<PoolWorker*> busyThreads;
        Lock listsLock;
        bool serving;
    
        class Job
        {
            public:
                PoolJob *job;
                void *data;
                Job(PoolJob *j, void *d) { job = j; data = d; };
        };
        
        std::list<Job> queue;
        
    public:
        ThreadPool(HTable conf);
        ~ThreadPool();

    public:
        void addJob(PoolJob *job, void *data = NULL, bool force = false);

    private:
        void markAsIdle(PoolWorker *worker);
        void queueJob(PoolJob *job, void *data);
        void workerTimedOut(PoolWorker *worker);
};


#endif

