#include <unistd.h>
#include <sys/types.h>
#ifndef WIN32
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#else
#include <winsock2.h>
#include <ws2tcpip.h>
#endif
#include "unisock.h"
#include "exceptions.h"
#include "log.h"


UnicastSocket::UnicastSocket(HTable params)
{
    int arg;
    
    sock = socket(AF_INET, SOCK_DGRAM, 0);
    if (sock < 0)
        throw MasException("Error creating socket.  Is TCP/IP installed? ;-)");

    arg = 1;
    if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (char*)&arg, sizeof(int)))
        masLog.warn("Error setting SO_REUSEADR");

    std::string bindAddr = std::string(table_get_strs(params, "bindaddress", "", NULL));
    if (bindAddr != "")
        sin.sin_addr.s_addr = inet_addr(bindAddr.c_str());
    else
        sin.sin_addr.s_addr = INADDR_ANY;
    addrLen = sizeof(struct sockaddr_in);
    
    if (bind(sock, (struct sockaddr*)&sin, addrLen) < 0)
    {
        shutdown(sock, 2);
        close(sock);
        throw MasException("Error binding unicast socket");
    } 
    
    std::string targetAddr = std::string(table_get_strs(params, "address", "", NULL));
    if (targetAddr != "")
        sin.sin_addr.s_addr = inet_addr(targetAddr.c_str());
    else
        sin.sin_addr.s_addr = INADDR_ANY;
}


UnicastSocket::~UnicastSocket()
{
    shutdown(sock, 2);
#ifdef WIN32
    closesocket(sock);
#else
    close(sock);
#endif
}
    
int UnicastSocket::read(unsigned char* buf, int max_size)
{
    return recv(sock, (char*)buf, max_size, 0);
}

int UnicastSocket::write(unsigned char* buf, int size)
{
    int sended  = sendto(sock, (char*)buf, size, 0, (struct sockaddr*)&sin, addrLen);
    return sended;
}

