#include "unit.h"
#include "exceptions.h"


Unit::Unit()
{
    outputBufferSize = 10240;
    output = NULL;
}


Unit::Unit(HTable conf)
{
    if (conf)
    {
        name = std::string(table_get_strs(conf, "name", "", NULL));
        outputBufferSize = table_get_int(conf, "outbuf", 10240, NULL);
        streamName = std::string(table_get_strs(conf, "descr", "", NULL));
        streamGenre = std::string(table_get_strs(conf, "genre", "", NULL));
        streamUrl = std::string(table_get_strs(conf, "url", "", NULL));
    }
    output = NULL;
}


Unit::~Unit()
{
}


int Unit::getInputsCount()
{
    return inputs.size();
}

void Unit::addInputLink(Link *link)
{
    if (! link) return;
    inputs.push_back(link);
    //link->addOutputRef();
}


void Unit::delInputLink(Link *link)
{
    if (! link) return;
    inputs.remove(link);
    //link->decOutputRef();
}


void Unit::setOutputLink(Link *link)
{
    output = link;
}


void Unit::removeOutputLink()
{
    output = NULL;
}


std::string Unit::getName()
{
    return name;
}


void Unit::setName(const std::string &name)
{
    this->name = name;
}


Link* Unit::getOutputLink()
{
    return output;
}


int Unit::getOutputBufferSize()
{
    return outputBufferSize;
}


void Unit::setOutputBufferSize(int bufferSize)
{
    outputBufferSize = bufferSize;
}


void Unit::save(HTable conf)
{
    table_set_str(conf, "type", getType().c_str());
    table_set_str(conf, "name", name.c_str());
    table_set_int(conf, "outbuf", outputBufferSize);
}

        
void Unit::patchMediaInfo(MediaInfo &minfo)
{
    if (streamName != "")
        minfo.setProperty("name", streamName);
    if (streamGenre != "")
        minfo.setProperty("genre", streamGenre);
    if (streamUrl != "")
        minfo.setProperty("url", streamUrl);
}

std::string Unit::getStreamName()
{
    return streamName;
}

void Unit::setStreamName(const std::string &s)
{
    streamName = s;
}

std::string Unit::getStreamGenre()
{
    return streamGenre;
}

void Unit::setStreamGenre(const std::string &s)
{
    streamGenre = s;
}

std::string Unit::getStreamUrl()
{
    return streamUrl;
}

void Unit::setStreamUrl(const std::string &s)
{
    streamUrl = s;
}


void SingleInputUnit::addInputLink(Link *link)
{
    if ((input) || (inputs.size() > 0))
        throw MasException("HttpOutput input slots full");
    Unit::addInputLink(link);
    input = link;
}

void SingleInputUnit::delInputLink(Link *link)
{
    if ((! link) || (input != output) || (inputs.size() != 1))
        throw MasException("Invalid link");
    Unit::delInputLink(link);
    input = NULL;
}

