#ifndef __UNIT_H__
#define __UNIT_H__


#include <set>
#include <list>
#include <string>

#include "thread.h"
#include "mediatype.h"
#include "link.h"
#include "conf.h"


class COREIMPORT Unit: public Thread
{
    private:
        int outputBufferSize;
    
    protected:
        Link *output;
        std::string name;
        std::list<Link*> inputs;
        std::string streamName;
        std::string streamGenre;
        std::string streamUrl;
    
    public:
        Unit();
        Unit(HTable conf);
        virtual ~Unit();

    public:
        virtual void save(HTable conf);
        virtual void patchMediaInfo(MediaInfo &minfo);
        
    public:
        virtual bool hasInputs() = 0;
        virtual int getMaxInputs() = 0;  // -1 for unlimites
        virtual bool hasOutputs() = 0;
        virtual std::set<MediaType> getAcceptedMediaTypes() = 0;
        virtual std::string getType() = 0;
        virtual int getInputsCount();

    public:
        int getOutputBufferSize();
        void setOutputBufferSize(int bufferSize);
        virtual void addInputLink(Link *link);
        virtual void delInputLink(Link *link);
        virtual void setOutputLink(Link *link);
        virtual Link* getOutputLink();
        virtual void removeOutputLink();

    public:
        virtual std::string getName();
        virtual void setName(const std::string &name);
        virtual std::string getStreamName();
        virtual void setStreamName(const std::string &streamName);
        virtual std::string getStreamGenre();
        virtual void setStreamGenre(const std::string &streamGenre);
        virtual std::string getStreamUrl();
        virtual void setStreamUrl(const std::string &streamUrl);
};


class COREIMPORT SingleInputUnit: public Unit
{
    protected:
        Link *input;
    
    public:
        SingleInputUnit(): Unit() { input = NULL; };
        SingleInputUnit(HTable conf): Unit(conf) { input = NULL; };
        virtual ~SingleInputUnit() { };
        
    public:
        virtual bool hasInputs() { return true; };
        virtual int getMaxInputs() { return 1; };
        virtual bool hasOutputs() { return false; };
        virtual void addInputLink(Link *link);
        virtual void delInputLink(Link *link);
};


class COREIMPORT SingleInputOutputUnit: public SingleInputUnit
{
    public:
        SingleInputOutputUnit(): SingleInputUnit() { };
        SingleInputOutputUnit(HTable conf): SingleInputUnit(conf) { };
        virtual ~SingleInputOutputUnit() { };
        
    public:
        virtual bool hasOutputs() { return true; };
};


class COREIMPORT OutputUnit: public Unit
{
    public:
        OutputUnit(): Unit() { };
        OutputUnit(HTable conf): Unit(conf) { };
        virtual ~OutputUnit() { };
        
    public:
        virtual bool hasInputs() { return false; };
        virtual int getMaxInputs() { return 0; };
        virtual bool hasOutputs() { return true; };
};


class COREIMPORT UnitFactory
{
    public:
        virtual ~UnitFactory() { };
        virtual Unit* createUnit(const std::string &name, HTable conf) = 0;
};


#endif

