#include "fileformats.h"


FileFormat::FileFormat(const std::string &filename):
            stream(filename.c_str(), std::ios::in | std::ios::binary)
{
    if (! stream.good())
        throw MasException("Error opening file '" + fullName + "'");
    fullName = filename;
}

FileFormat::~FileFormat()
{
    stream.close();
}



FileFormatRegistry::FileFormatRegistry()
{
}

FileFormatRegistry::~FileFormatRegistry()
{
    for (std::list<FileFormatFactory*>::iterator i = formats.begin();
            i != formats.end(); i++)
        delete *i;
}


FileFormat* FileFormatRegistry::createFileFormat(const std::string &filename)
{
    FileFormatFactory *factory;

    std::ifstream ifs(filename.c_str(), std::ios::in | std::ios::binary);
    if (! ifs.good())
        throw MasException("Error opening file '" + filename + "'");
    
    std::string ext = getFileExtLower(filename);
    if (ext != "")
    {
        for (std::list<FileFormatFactory*>::iterator i = formats.begin();
                i != formats.end(); i++)
        {
            factory = *i;
            if (factory->isOurFileByExt(ext))
            {
                if (factory->isOurFileByContent(ifs, filename, ext))
                    return factory->createFileFormat(filename);
            }
        }
    }

    for (std::list<FileFormatFactory*>::iterator i = formats.begin();
            i != formats.end(); i++)
    {
        factory = *i;
        if (factory->isOurFileByContent(ifs, filename, ext))
            return factory->createFileFormat(filename);
    }
 
    ifs.close();
    throw MasException("Unsupported file format '" + filename + "'");
}


void FileFormatRegistry::registerFactory(FileFormatFactory *factory)
{
    formats.push_back(factory);
}


extern "C" int EXPORT_REGFUNC register_fileformats_plugins()
{
    int res = 0;
    try
    {
        Server::getServer()->addPlugin(std::string("fileformats"), 
                new FileFormatRegistry());
    }
    catch (MasException &e)
    {
        masLog.error(e.getMessage());
        res = 1;
    }
    catch (...)
    {
        res = 1;
    }
    return res;
}

