#ifndef __FILEFORMATS_H__
#define __FILEFORMATS_H__


#include <fstream>
#include <string>
#include <list>
#include "maserver.h"


#ifdef WIN32
#ifdef BUILDING_FILEFORMATS
#define FORMATIMPORT __declspec (dllexport)
#else
#define FORMATIMPORT __declspec (dllimport)
#endif
#else
#define FORMATIMPORT
#endif


class FORMATIMPORT FileFormat
{
    protected:
        std::string fullName;
        std::ifstream stream;
        
    public:
        FileFormat(const std::string &filename);
        virtual ~FileFormat();
        virtual void packetize(Interruptable *intr, PacketizerReceiver *receiver) = 0;
        virtual bool hasMediaInfo() = 0;
        virtual MediaInfo getMediaInfo() = 0;
};


class FORMATIMPORT FileFormatFactory
{
    public:
        virtual ~FileFormatFactory() { };
        virtual bool isOurFileByExt(const std::string &fileExtension) = 0;
        virtual bool isOurFileByContent(std::ifstream &stream,
                const std::string &fullname, const std::string &ext) = 0;
        virtual FileFormat* createFileFormat(const std::string &fullname) = 0; 
};


class FORMATIMPORT FileFormatRegistry: public Plugin
{
    private:
        std::list<FileFormatFactory*> formats;
        
    public:
        FileFormatRegistry();
        virtual ~FileFormatRegistry();

    public:
        FileFormat* createFileFormat(const std::string &filename);
        void registerFactory(FileFormatFactory *factory);
};


#endif

