extern "C" {
#include <lua40/lua.h>
#include <lua40/lauxlib.h>
}
#include "luapllist.h"
#include "exceptions.h"


static void call_void_lua_function(lua_State *L, const std::string& name)
{
    int oldTop = lua_gettop(L);
    lua_pushstring(L, name.c_str());
    lua_call(L, 0, 0);
    lua_settop(L, oldTop);
}



static std::string call_str_lua_function(lua_State *L, const std::string& name)
{
    int oldTop = lua_gettop(L);
    lua_pushstring(L, name.c_str());
    lua_call(L, 0, 1);
    const char *s = lua_tostring(L, 0);
    lua_settop(L, oldTop);
    if (! s)
        return "";
    else
        return std::string(s);
}



LuaPlaylistHandler::LuaPlaylistHandler(HTable table)
{
    std::string filename;
    
    lua = lua_open(0);
    if (! lua)
        throw MasException("Error creating lua virtual mashine");

    filename = std::string(table_get_strs(table, "file", "", NULL));
    if (filename == "")
        throw MasException("Lua script filename not specified");
    initLuaVm(table_get_table(table, "parameters", NULL, NULL));
    int oldtop = lua_gettop(lua);
    if (lua_dofile(lua, filename.c_str()))
    {
        lua_close(lua);
        throw MasException("Error loading lua playlist module");
    }
    lua_settop(lua, oldtop);
    call_void_lua_function(lua, "init_playlist");
}


LuaPlaylistHandler::~LuaPlaylistHandler()
{
    call_void_lua_function(lua, "done_playlist");
    lua_close(lua);
}


void LuaPlaylistHandler::initLuaVm(HTable parameters)
{
   /* lua_baselibopen(lua);
    lua_iolibopen(lua);
    lua_strlibopen(lua);
    lua_mathlibopen(lua);
    lua_dblibopen(lua);*/

  /*  lua_pushstring(lua, "parameters"); */
    lua_newtable(lua);
    lua_setglobal(lua, "parameters");
    lua_getglobal(lua, "parameters");
   /* lua_settable(lua, LUA_GLOBALSINDEX);
    lua_pushstring(lua, "parameters");
    lua_gettable(lua, LUA_GLOBALSINDEX);*/
    
    if (parameters)
    {
        HTableIterator iter = table_get_iter(parameters);
        if (iter)
        {
            while (table_iter_next(iter))
            {
                lua_pushstring(lua, table_iter_get_name(iter));
                switch (table_iter_get_type(iter))
                {
                    case TYPE_INT:
                        lua_pushnumber(lua, table_iter_get_int(iter, NULL));
                        lua_settable(lua, -2);
                        break;
                    
                    case TYPE_FLOAT:
                        lua_pushnumber(lua, table_iter_get_double(iter, NULL));
                        lua_settable(lua, -2);
                        break;

                    case TYPE_STRING:
                        lua_pushstring(lua, table_iter_get_strs(iter, NULL));
                        lua_settable(lua, -2);
                        break;

                    default:
                        throw MasException("Unsupported type of lua parameter");
                }
            }
            table_free_iter(iter);
        }
    }

    lua_pop(lua, 1);
}


std::string LuaPlaylistHandler::getNextSong()
{
    return call_str_lua_function(lua, "get_next_song");
}



PlaylistHandler* LuaPlaylistFactory::createPlaylist(HTable args)
{
    if (args)
        return new LuaPlaylistHandler(args);
    else
        throw MasException("Can't create LUA playlist handler without parameters");
}


extern "C" int EXPORT_REGFUNC register_luaplist_plugins()
{
    int res = 0;
    try
    {
        Server::getServer()->addPlugin("playlist::lua", 
                new LuaPlaylistFactory());
    }
    catch (MasException &e)
    {
        masLog.error(e.getMessage());
        res = 1;
    }
    catch (...)
    {
        res = 1;
    }
    return res;
}

