#include <unistd.h>
#include <sys/time.h>
#include <BasicUsageEnvironment.hh>
#include <MP3FileSource.hh>
#include "httpinput.h"
#include "exceptions.h"
#include "mp3.h"
#include "log.h"
#include "packetsink.h"


#define MAX_FRAME_SIZE 40960



HttpInput::HttpInput(HTable config, PacketBuffer *buf): InputProtocol(config, buf)
{
    if (! config)
        throw MasException("Http input config missing");
    maxFrameSize = MAX_FRAME_SIZE;
    frameBuf = new unsigned char[maxFrameSize];
}


HttpInput::~HttpInput()
{
    delete[] frameBuf;
}


void HttpInput::afterPlay()
{
    exitFrameLoop = 1;
}

static void afterPlayingWrapper(void* data)
{
    FileInput* o = (FileInput*)data;
    o->afterPlay();
}


void HttpInput::run()
{
    MP3HttpSource *source = NULL;
    PacketBufferSink *sink = NULL;
    BasicTaskScheduler scheduler;
    BasicUsageEnvironment env(scheduler);
    sink = PacketBufferSink::createNew(env, buffer);
    if (! sink)
        throw MasException("Error creating PacketBuferSink");
    
    while (! isShuttingDown())
    {
        try
        {
            source = MP3HttpSource::createNew(env, addr, port, hostName, location);
            if (! source)
                throw MasException("Error creating HTTP source, '" + hostName + "'");

            sink->startPlaying(*source, afterPlayingWrapper, (void*)this);
            exitFrameLoop = 0;
            while (! exitFrameLoop)
            {
                scheduler.SingleStep();
                sink->continuePlaying();
//std::cout << "next step" << std::endl;
                if (isShuttingDown())
                    exitFrameLoop = 1;
            }
            
            Medium::close(source);
            source = NULL;
        }
        catch (MasException& e)
        {
            log_message(LOG_ERROR, e.getMessage());
            if (source)
                Medium::close(source);
        }
        catch (...)
        {
            log_message(LOG_ERROR, "Error playing file");
            if (source)
                Medium::close(source);
        }
    }
    Medium::close(sink);
}

