#ifndef __HTTP_OUTPUT_H__
#define __HTTP_OUTPUT_H__


#include <pthread.h>
#include "maserver.h"
#include "httpd.h"



class HttpOutput;


class HttpPeer: public PacketizerReceiver, public Interruptable
{
    private:
        HttpdConnection *connection;
        HttpOutput *output;
        Link *link;
        bool useIcyMetadata;
        int bytesLeftBeforeMeta;
        std::string introFilename;
        char block;
        struct timeval curTime;
        MediaInfo minfo;
    
    public:
        HttpPeer(Link *link, HttpdConnection *conn, HttpOutput *out,
                const std::string& onConnect);
        virtual ~HttpPeer();

    public:
        void run();
        virtual void addData(unsigned char *data, int length, double playTime,
                const MediaType &mediaType);

    private:
        void sendHeader();
        void sendIcyMeta();
        void playIntro();
        void sendData(unsigned char *data, int length);
};


class HttpOutput: public SingleInputUnit, public HttpdHandler
{
    private:
        std::string mount;
        std::string onConnect;
        Httpd *httpd;
   
    public:
        HttpOutput();
        HttpOutput(HTable params);
        virtual ~HttpOutput();

    public:
        virtual std::set<MediaType> getAcceptedMediaTypes();
        virtual std::string getType() { return "httpoutput"; };
        virtual void run();
        int handleRequest(HttpdConnection& connection, const ServerURL& url, 
                enum RequestType requestType);
        std::string getStreamTitle();
};


class HttpOutputFactory: public UnitFactory
{
    public:
        HttpOutputFactory() { };
        virtual ~HttpOutputFactory() { };
        virtual Unit* createUnit(const std::string &name, HTable conf);
};

#endif

