#include "decoder.h"


ExternalMp3Decoder::ExternalMp3Decoder(const std::string &prog, 
        PacketizerReceiver *recv)
{
    program = prog;
    receiver = recv;
}

ExternalMp3Decoder::~ExternalMp3Decoder()
{
}

void ExternalMp3Decoder::addData(unsigned char *data, int length, double playTime, 
        const MediaType &mediaType)
{
}


ExternalMp3DecoderFactory::ExternalMp3DecoderFactory(const std::string &prog)
{
    program = prog;
}

bool ExternalMp3DecoderFactory::isSupportedMedia(const MediaType &mediaType)
{
    if ((mediaType.getType() == "audio") && (mediaType.getSubtype() == "mp3"))
        return true;
    else
        return false;
}


Decoder* ExternalMp3DecoderFactory::createDecoder(const MediaType &mediaType,
        PacketizerReceiver *receiver, HTable params)
{
    if (isSupportedMedia(mediaType))
        return new ExternalMp3Decoder(program, receiver);
    else
        throw MasException("Invalid libmpg123 media type '" +
                mediaType.getType() + "/" + mediaType.getSubtype() + "'");
}



ExtMp3DecoderLoader::ExtMp3DecoderLoader()
{
    registry = (DecoderRegistry*)Server::getServer()->findPlugin("decoderRegistry");
    if (! registry)
        throw MasException("decoderRegistry plugin not found");
}

void ExtMp3DecoderLoader::load(const std::string& path, const std::string& name, 
        HTable descr)
{
    std::string program(table_get_strs(descr, "run", "", NULL));
    if (program == "")
        throw MasException("Invalid external mp3 decoder '" + name + "'");
    registry->registerDecoder(name, new ExternalMp3DecoderFactory(program));
}



extern "C" int EXPORT_REGFUNC register_extmp3dec_plugins()
{
    int res = 0;
    try
    {
        Server::getServer()->addLoader("extmp3dec", new ExtMp3DecoderLoader());
    }
    catch (MasException &e)
    {
        masLog.error(e.getMessage());
        res = 1;
    }
    catch (...)
    {
        res = 1;
    }
    return res;
}

