#include <stdio.h>
#include <string.h>
#include <iostream>
#include "maserver.h"
#include "mp3packetizer.h"


#define MPEG_MODE_MONO 3


static const unsigned int bitrate[3][3][16] =
{
    {
        { 0, 32, 64, 96, 128, 160, 192, 224, 256, 288, 320, 352, 384, 416, 448, 0 },
        { 0, 32, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 384, 0 },
        { 0, 32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 0 }
    }, 
    
    {
        { 0, 32, 48, 56, 64, 80, 96, 112, 128, 144, 160, 176, 192, 224, 256, 0 },
        { 0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160, 0 },
        { 0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160, 0 }
    }, 
    
    {
        { 0, 32, 48, 56, 64, 80, 96, 112, 128, 144, 160, 176, 192, 224, 256, 0 },
        { 0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160, 0 },
        { 0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160, 0 }
    }
};



static const unsigned int samplerate[3][4] =
{
	{ 44100, 48000, 32000, 0 },
	{ 22050, 24000, 16000, 0 },
	{ 11025, 8000, 8000, 0 }
};



MP3Packetizer::MP3Packetizer(PacketizerReceiver *r): Packetizer(r)
{
    bufSize = 2100;
    buffer = new unsigned char[bufSize];
    clearState();
    mtype = NULL;
}


MP3Packetizer::~MP3Packetizer()
{
    delete[] buffer;
}


static int parseFrameHeader(unsigned long header, MP3Header& mh)
{
    mh.syncword = (header >> 20) & 0x0fff;
    mh.version = ((header >> 19) & 0x01) ? 0 : 1;
    mh.layer = 3 - ((header >> 17) & 0x03);
    if (mh.layer != 2)
        return -1;
    mh.errorProtection = ((header >> 16) & 0x01) ? 0 : 1;
    mh.bitrateIndex = (header >> 12) & 0x0F;
    mh.samplerateIndex = (header >> 10) & 0x03;
    mh.padding = (header >> 9) & 0x01;
    mh.extension = (header >> 8) & 0x01;
    mh.mode = (header >> 6) & 0x03;
    mh.modeExt = (header >> 4) & 0x03;
    mh.copyright = (header >> 3) & 0x01;
    mh.original = (header >> 2) & 0x01;
    mh.emphasis = header & 0x03;

    mh.stereo = (mh.mode == MPEG_MODE_MONO) ? 1 : 2;
    if ((mh.version < 0) || (mh.version > 2))
        return -1;
    if ((mh.layer < 0) || (mh.layer > 2))
        return -1;
    if ((mh.bitrateIndex < 0) || (mh.bitrateIndex > 15))
        return -1;
    if ((mh.samplerateIndex < 0) || (mh.samplerateIndex > 3))
        return -1;
    mh.bitrate = bitrate[mh.version][mh.layer][mh.bitrateIndex];
    if (mh.bitrate == 0)
        return -1;
    mh.samplerate = samplerate[mh.version][mh.samplerateIndex];
    if (mh.samplerate == 0)
        return -1;
    
    if (mh.version == 0)
        mh.samples = 1152;
    else
        mh.samples = 576;

    if (mh.samplerate)
    {
        mh.framesize = (int)(((float)mh.samples * mh.bitrate * 1000 / 
                (float)mh.samplerate) / 8 + mh.padding);
        int fs = (144000 * mh.bitrate / mh.samplerate) + mh.padding;
if (fs != mh.framesize)
masLog.debug("framesize: " + numToStr(mh.framesize) + " " + numToStr(fs));
        mh.framesize = fs;
    }
    else
        return -1;

    if ((mh.framesize < 300) || (mh.framesize > 2048))
        return -1;

    return 0;
}


void MP3Packetizer::checkForHeader()
{
    unsigned long int header = 0L;
    unsigned char ch;
    int pos = bufEnd - 4;
    if (pos < 0)
        pos += bufSize;
    for (int i = 0; i < 4; i++)
    {
        ch = buffer[pos];
        header = (header << 8) | ch;
        pos++;
        if (pos >= bufSize)
            pos -= bufSize;
    }
    if (! parseFrameHeader(header, mh))
    {
        if (bufEnd != 4)
        {
            int start = bufEnd - 4;
            if (start < 0)
                start += bufSize;
            if (start + 4 <= bufSize)
                memmove(buffer, buffer + start, 4);
            else
            {
                int split = bufSize - start;
                memcpy(buffer + split, buffer, 4 - split);
                memcpy(buffer, buffer + start, split);
            }
            bufEnd = 4;
        }
        bufStart = 0;
        filled = 4;
        findStart = false;
        waitForData = true;
        dataRemaining = mh.framesize - 4;
    }
}


void MP3Packetizer::clearState()
{
    filled = 0;
    bufStart = bufEnd = 0;
    findStart = true;
    waitForData = false;
}


void MP3Packetizer::releasePacket()
{
    if (mtype)
    {
        if ((mtype->getBitrate() != mh.bitrate) || 
                (mtype->getSamplerate() != mh.samplerate) || 
                (mtype->getStereo() != mh.stereo))
        {
            delete mtype;
            mtype = NULL;
        }
    }
    if (! mtype)
        mtype = new MP3MediaType(mh.bitrate, mh.samplerate, mh.stereo);
    
    double playTime = (double)mh.samples / (double)mh.samplerate * (double)1000000.0;
    double pt = ((10000.0 * 1152.0) / ((double)mh.samplerate / 100.0));
if (pt != playTime)
masLog.debug("playTime: " + numToStr(playTime) + " " + numToStr(pt));
    recv->addData(buffer, mh.framesize, pt, *mtype);
    clearState();
}


void MP3Packetizer::addData(unsigned char ch)
{
    buffer[bufEnd] = ch;
    filled++;
    if (filled > bufSize)
    {
        bufStart++;
        if (bufStart >= bufSize)
            bufStart -= bufSize;
        filled = bufSize;
    }

    bufEnd++;
    if (bufEnd >= bufSize)
        bufEnd -= bufSize;

    if (findStart && (filled >= 4))
        checkForHeader();
    else
        if (waitForData)
        {
            dataRemaining--;
            if (dataRemaining < 1)
                releasePacket();
        }
}


void MP3Packetizer::addData(unsigned char *data, int length)
{
    int i = 0;
    int sz;

    while (i < length)
    {
        if (findStart)
        {
            addData(data[i]);
            i++;
        }
        else
        {
            if (dataRemaining <= length - i)
                sz = dataRemaining;
            else
                sz = length - i;
            memcpy(buffer + bufEnd, data + i, sz);
            bufEnd += sz;
            filled += sz;
            dataRemaining -= sz;
            if (dataRemaining < 1)
                releasePacket();
            i += sz;
        }
    }
}


Packetizer* MP3PacketizerFactory::createPacketizer(const std::string &name, 
        PacketizerReceiver *r)
{
    if (name == "mp3")
        return new MP3Packetizer(r);
    else
        throw MasException("Invalid type '" + name + "'in mp3 packetizer factrory");
}


extern "C" int EXPORT_REGFUNC register_mp3packetizer_plugins()
{
    int res = 0;
    try
    {
        Server::getServer()->addPacketizerFactory(std::string("mp3"), 
                new MP3PacketizerFactory());
    }
    catch (MasException &e)
    {
        masLog.error(e.getMessage());
        res = 1;
    }
    catch (...)
    {
        res = 1;
    }
    return res;
}

