#include "mp3rtpproducer.h"


Mp3RtpProducer::Mp3RtpProducer(RtpSender *asender, int amaxBufSize)
{    
    sender = asender;
    if (! sender)
        throw MasException("Invalid sender in mp3 rtp producer");
    maxBufSize = amaxBufSize;
    buffer = new unsigned char[maxBufSize];
    filled = 0;
    packetsCount = 0;
    packetsTime = 0.0;
}


Mp3RtpProducer::~Mp3RtpProducer()
{
    if (buffer) delete[] buffer;
}


void Mp3RtpProducer::makePackets(unsigned char *data, int length, 
        double playTime, const MediaType &mediaType)
{
    if (length > maxBufSize)
        throw MasException("RTP packet too large"); /// TODO: fragment it
   
    if (length + filled > maxBufSize)
        flush();
    
    memcpy(buffer + filled, data, length);
    filled += length;
    packetsCount++;
    packetsTime += playTime;
    if (length == maxBufSize)
        flush();
}

void Mp3RtpProducer::flush()
{
    if ((filled < 1) || (packetsCount < 1))
        return;
    sender->sendData(buffer, filled, packetsTime, 0x0E);
    filled = 0;
    packetsCount = 0;
    packetsTime = 0.0;
}


bool Mp3RtpProducerFactory::isSupportedMediaType(const MediaType &mediaType)
{
    if ((mediaType.getType() == "audio") && (mediaType.getSubtype() == "mp3"))
        return true;
    else
        return false;
}

RtpPacketProducer* Mp3RtpProducerFactory::createPacketProducer(RtpSender *sender,
        const MediaType &mediaType)
{
    if (! isSupportedMediaType(mediaType))
        throw MasException("Creating mp3 rtp producer for unsupported media type");
    return new Mp3RtpProducer(sender);
}


extern "C" int EXPORT_REGFUNC register_mp3rtpproducer_plugins()
{
    int res = 0;
    try
    {
        RtpPacketProducerRegistry *registry = (RtpPacketProducerRegistry*)
            Server::getServer()->findPlugin("rtpPacketProducerRegistry");
        if (! registry)
            throw MasException("rtpPacketProducerRegistry plugin not found");
        registry->registerFactory(new Mp3RtpProducerFactory());
    }
    catch (MasException &e)
    {
        masLog.error(e.getMessage());
        res = 1;
    }
    catch (...)
    {
        res = 1;
    }
    return res;
}

