#ifndef __RTP_INPUT_H__
#define __RTP_INPUT_H__


#include "maserver.h"
#include "rtp.h"


class RtpInput: public OutputUnit, public PacketizerReceiver
{
    private:
        unsigned char* frameBuf;
        unsigned long long int startTime;
        double playingTime;
        int maxFrameSize;
        ReadSocket *rtpSock;
        ReadSocket *annSock;
        RtpPacketParser *parser;
        RtpPacketParserRegistry *registry;
        int payload;
        
    public:
        RtpInput();
        RtpInput(HTable params);
        virtual ~RtpInput();

    public:
        virtual std::set<MediaType> getAcceptedMediaTypes();
        virtual std::string getType() { return std::string("rtpinput"); };
        virtual void addData(unsigned char *data, int length, double playTime, 
                const MediaType &mediaType);
        virtual void run();

    private:
        void readRtpPacket();
        void readAnnPacket();
        void processHeader(const std::string& name, const std::string& value);
        void parseHeader(const std::string& line);
};


class RtpInputUnitFactory: public UnitFactory
{
    public:
        RtpInputUnitFactory() { };
        virtual ~RtpInputUnitFactory() { };
        virtual Unit* createUnit(const std::string &name, HTable conf);
};


#endif

