#ifndef __RTP_H__
#define __RTP_H__


#include "maserver.h"


#ifdef WIN32
#ifdef BUILDING_RTP
#define RTPIMPORT __declspec (dllexport)
#else
#define RTPIMPORT __declspec (dllimport)
#endif
#else
#define RTPIMPORT
#endif


class RTPIMPORT RtpPacketProducerRegistry;
class RTPIMPORT RtpPacketProducer;


class RTPIMPORT RtpSender
{
    private:
        WriteSocket& wsock;
        int ssrc;
        unsigned char *buffer;
        int maxPacketSize;
        int headerGap;
        int sequenceNo;
        unsigned int rtpTimestamp;
        MediaType *mediaType;
        RtpPacketProducer *producer;
        RtpPacketProducerRegistry *registry;
    
    public:
        RtpSender(WriteSocket& wsock, int packetSize=1500);
        virtual ~RtpSender();

    public:
        void sendPacket(unsigned char *data, int length, double playTime, 
                const MediaType &mediaType);
        void flush();
        void sendData(unsigned char *data, int length, double playTime, 
                int payload);

    private:
        void makePacket(int payload, double playTime);
};


class RTPIMPORT RtpPacketProducer
{
    public:
        virtual ~RtpPacketProducer() { };
        virtual void makePackets(unsigned char *data, int length, 
                double playTime, const MediaType &mediaType) = 0;
        virtual void flush() = 0;
};


class RTPIMPORT RtpPacketProducerFactory
{
    public:
        virtual ~RtpPacketProducerFactory() { };
        virtual bool isSupportedMediaType(const MediaType &mediaType) = 0;
        virtual RtpPacketProducer* createPacketProducer(RtpSender *sender,
                const MediaType &mediaType) = 0;
};


class RTPIMPORT RtpPacketProducerRegistry: public Plugin
{
    private:
        std::list<RtpPacketProducerFactory*> factories;
        
    public:
        RtpPacketProducerRegistry() { };
        virtual ~RtpPacketProducerRegistry();
        void registerFactory(RtpPacketProducerFactory *factory);
        RtpPacketProducer* createProducer(RtpSender *sender, 
                const MediaType &mediaType);
};


class RTPIMPORT RtpPacketParser
{
    public:
        virtual ~RtpPacketParser() { };
        virtual void parsePacket(unsigned char *data, int length) = 0;
        virtual void flush() = 0;
};


class RTPIMPORT RtpPacketParserFactory
{
    public:
        virtual ~RtpPacketParserFactory() { };
        virtual RtpPacketParser* createParser(int payload, PacketizerReceiver *recv) = 0;
};


class RTPIMPORT RtpPacketParserRegistry: public Plugin
{
    private:
        std::map<int, RtpPacketParserFactory*> factories;
        
    public:
        RtpPacketParserRegistry() { };
        virtual ~RtpPacketParserRegistry();
        void registerFactory(int payload, RtpPacketParserFactory *factory);
        RtpPacketParser* createParser(int payload, PacketizerReceiver *recv);
};


#endif

