#include <time.h>
#include <unistd.h>
#include <fstream>
#include "shoutoutput.h"
#include "exceptions.h"
#include "servconf.h"
#include "log.h"
#include "mp3.h"


ShoutOutput::ShoutOutput(HTable params, PacketBuffer *buf, InputProtocol *in): 
                OutputProtocol(params, buf, in)
{
    addr = std::string(table_get_strs(params, "address", "127.0.0.1", NULL));
    port = table_get_int(params, "port", 8000, NULL);
    passwd = std::string(table_get_strs(params, "password", "", NULL));
    mount = std::string(table_get_strs(params, "mount", "", NULL));

    shout_init_connection(&conn);
    
    std::string proto = std::string(table_get_strs(params, "proto", "xaudiocast", NULL));
    if (proto == "icy")
        conn.icy_compat = 1;
    else if (proto != "xaudiocast")
        throw MasException("Invalid shout protocol type");

    name = input->getStreamName();
    url = input->getStreamUrl();
    genre = input->getStreamGenre();
    descr = input->getStreamDescr();
    
    conn.ip = (char*)addr.c_str();
    conn.port = port;
    conn.password = (char*)passwd.c_str();
    conn.mount = (char*)mount.c_str();
    conn.name = (char*)name.c_str();
    conn.url = (char*)url.c_str();
    conn.description = (char*)descr.c_str();
    conn.genre = (char*)genre.c_str();
    
    if (! shout_connect(&conn)) 
        throw MasException("Error connecting to shoutcast server");
}


ShoutOutput::~ShoutOutput()
{
    shout_disconnect(&conn);
}


void ShoutOutput::updateMetadata()
{
    std::string metadata;
    std::string title = input->getSongTitle();
    std::string artist = input->getSongArtist();
    
    if ((title != "") && (artist != ""))
        metadata = artist + " - " + title;
    else
        if (title != "")
            metadata = title;
        else
            if (artist != "")
                metadata = artist;
    
    shout_update_metadata(&conn, (char*)metadata.c_str());
}


#define UPDATE_TITLE_RATE 30


void ShoutOutput::run()
{
    unsigned char *buf;
    int bufSize = 1024;
    int res;
    unsigned long long int packetNo = 0;
    time_t last_update = 0, now;

    buf = (unsigned char*)malloc(bufSize);
    if (! buf)
        throw MasException("Error allocating buffer");

    while (! isShuttingDown())
    {
//std::cout << "getting data" << std::endl;
        res = buffer->getData(buf, bufSize, packetNo, true);
        if (res > bufSize)
        {
//std::cout << "buffer too small" << std::endl;
            unsigned char *b = (unsigned char*)realloc(buf, res);
            if (! b)
            {
                free(buf);
                throw MasException("Error sending to shout: packet too big");
            }
            else
                buf = b;
            res = buffer->getData(buf, bufSize, packetNo, false, true);
            buffer->unlock();
        }
//std::cout << "ok" << std::endl;
       
        if (res <= 0)
        {
            log_message(LOG_ERROR, "Error getting data for shout from PacketBuffer"); 
        }
        else
        {
            if (! shout_send_data(&conn, buf, res))
            {
                log_message(LOG_ERROR, "Error %i sending file to shout server", 
                        conn.error);
                break;
            } 
            else 
            {
                now = time(NULL);
                if (now - last_update >= UPDATE_TITLE_RATE)
                {
                    updateMetadata();
                    last_update = now;
                }
                
            }
        }
        //shout_sleep(&conn);
    }
    free(buf);
}

